@@ -201,6 +201,17 @@ class Task:
         # convert self.ranks to rank conditional object:
         self.ranks = rank_conditional(self.ranks)
 
+        if "*" in self.before and self.after:
+            raise ValueError(
+                f"{self.name}: cannot specify 'before=[\"*\"]' "
+                f"when 'after' is also set (after={self.after})"
+            )
+        if "*" in self.after and self.before:
+            raise ValueError(
+                f"{self.name}: cannot specify 'after=[\"*\"]' "
+                f"when 'before' is also set (before={self.before})"
+            )
+
     @staticmethod
     def _check_spec(spec, getter):
         """
@@ -1020,8 +1031,12 @@ class Modprobe:
         for name in tasks:
             task = self.get_task(name)
             if "*" in task.after:
+                # Add all tasks to deps (except those that also specify "*"
+                # in their 'after' list)
                 deps[task.name] = [
-                    self.get_task(x).name for x in tasks if x != task.name
+                    self.get_task(x).name
+                    for x in tasks
+                    if "*" not in self.get_task(x).after
                 ]
             else:
                 after_tasks = [self.get_task(x).name for x in task.after]
